///////////////////////////////////////////////////////////////////////////////
// TSP102_Source.c
//
// Sample source for TSReader
//     Copyright (C) 2004-2005 COOLSTF.com Inc.
//
// Extended to support the Teleview TSP102 card for monitoring DVB ASI or
// SMPTE 310M.
//     02-21-2006 Robert Blazewicz at TV Guide On Screen
//
// Install:
// 1. Copy the 'TSPLLD0421.dll' file and the 'rbf' directory into the
//    TSReader\ directory.
// 2. Copy the 'TSReader_Teleview_TSP102_ASI.dll' and
//    'TSReader_Teleview_TSP102_310M.dll' files into the TSReader\Sources\
//    directory.
// 3. When TSReader uses this source a log file named 'TSPLOWA1.LOG' is
//    generated by the Teleview library to aid debugging.
///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "TSReader\inc\sources.h"
#include "Teleview\inc\lld_def.h"
#include "TSP102_Source.h"

PSOURCESTRUCT ss;
HANDLE hInstance;
char szLastSignalReport[128] = {0};        
char szLastTune[128] = {0};

// TSReader_Tune()
// 
// This is where you setup the tuner to receive the transport stream
// If you're a source with CAPABILITIES_SERIAL_CONTROL you should
// call into the SourceHelper_TuneSerialControl() function to have it
// tune the serially controlled receiver.
//
// Return TRUE if you locked, FALSE if no lock and -1 if you're a terrestrial tuner that detects analog signals

BOOL TSReader_Tune()
{
	return TRUE;
}

// This is another sample input data thread that can handle unaligned
// transport streams. The TSReader_SourceHelper.dll has three functions
// which allow you to send over non-aligned MPEG-2 transport stream data
// with or without Reed-Solomon codes and/or timestampts and then pass
// the aligned data into TSReader. This isn't as efficient as the prior
// sample thread but there's very little additional CPU load to handle
// stream alignment in software. 
//
// NOTE: This method is supported in version 2.6.42 and later.

DWORD WINAPI ReadDataThreadNonAligned(LPVOID lpv)
{
	// Actual play begin
	TSPL_SET_BANK_COUNTER(0);

	// Capture mode start
	TSPL_SET_SDCON_MODE(TSPL_SDCON_CAPTURE_MODE);

	// Wait to fill the on board buffer; first Bank Group is 0
	while (!ss->fTerminateReadThread && TSPL_GET_CUR_BANK_GROUP() == 0) {}

	SourceHelper_StartSyncThread(ss, FALSE);
	while (!ss->fTerminateReadThread)
	{
		int nIdxSubBank;
		const int nIdCurBank = TSPL_GET_CUR_BANK_GROUP();
		if (nIdCurBank == 0xffff)
			// Fail alarm; invalid bank value
			break;

		for (nIdxSubBank = 0; !ss->fTerminateReadThread && nIdxSubBank < TSP102_SUB_BANKS+1; nIdxSubBank++)
		{
			BYTE* bpDMABufLinearAddr = TSPL_READ_BLOCK();
			if (bpDMABufLinearAddr == NULL)
				// Fail to read block, abort
				break;

			SourceHelper_SyncData(bpDMABufLinearAddr, TSP102_SUB_BANK_KBYTES * 1024);
		}

		// Wait to fill next buffer
		while (!ss->fTerminateReadThread && TSPL_GET_CUR_BANK_GROUP() == nIdCurBank) {}
	}
	SourceHelper_StopSyncThread();

	// Time to quit - flag buffers in use as -1000 which tells
	// TSReader's main thread to terminate. If you don't do this
	// TSReader will hang when exiting.
	ss->fReadThreadTerminated = TRUE;
	ss->fTerminateReadThread = FALSE;
	EnterCriticalSection(&ss->csTSBuffersInUse);
	ss->nTSBuffersInUse = -1000;
	LeaveCriticalSection(&ss->csTSBuffersInUse);
	return 0;
}

// This function is called by TSReader when it wants to start the source so it can receive
// data. This gives you a chance to setup a thread to move the data. 
BOOL TSReader_Start()
{
	DWORD dwThreadID;

	ss->hReadDataThread = CreateThread(NULL, 0, ReadDataThreadNonAligned, (LPVOID)0, CREATE_SUSPENDED, &dwThreadID);
	SourceHelper_SetWorkerThreadPriorities(FALSE);
	ResumeThread(ss->hReadDataThread);

	return TRUE;
}

// Opposite of the start function - close down the thread before returning
BOOL TSReader_Stop()
{
	ss->fTerminateReadThread = TRUE;
	while (ss->fReadThreadTerminated == FALSE)
		Sleep(50);
	return TRUE;
}

// This function is called when TSReader first loads up the source. Use it to find and initialize
// your hardware. Return FALSE if your hardware didn't initialize.
BOOL TSReader_Init(PSOURCESTRUCT pss)
{
	ss = pss;

	TSPL_SET_TSIO_DIRECTION(TSP102_TSIO_FORMAT ? TSIO_ASI_LOOPTHRU : TSIO_310_LOOPTHRU);

	if (TSPL_SET_SDRAM_BANK_CONFIG(TSP102_SUB_BANKS) == -1)
		// FAIL: can not set SDRAM BANK mode, unstable board status
		return FALSE;

	if (TSPL_SET_SDRAM_BANK_OFFSET_CONFIG(TSP102_SUB_BANK_KBYTES) == -1)
		// FAIL: can not set SDRAM SUBBANK OFFSET mode, unstable board status
		return FALSE;

	TSPL_RESET_SDCON();

	return TRUE;
}

// Opposite of the Init() function
BOOL TSReader_DeInit()
{
	return TRUE;
}

// TSReader calls this function to get the tuner parameters. You can build your own dialog
// or use one of the TSReader standard ones exported in TSReader_SourceHelper.dll
BOOL TSReader_TuneDialog(HWND hWndParent)
{
#ifdef ATSC
		if (SourceHelper_ATSCTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif ATSC
#ifdef DVBS
		if (SourceHelper_DVBSTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif DVBS
#ifdef DVBT
		if (SourceHelper_DVBTTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif DVBT
#ifdef DVBC
		if (SourceHelper_DVBCTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif DVBC
#ifdef DSS
		if (SourceHelper_DSSTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif DSS
#ifdef ADVANCED_SATELLITE
		if (SourceHelper_ADVTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif ADVANCED_SATELLITE
#ifdef QAM
		if (SourceHelper_QAMTuneDialog(hWndParent) == FALSE)					return FALSE;
#endif QAM
#ifdef UDPMULTICAST
		if (SourceHelper_UDPMulticastTuneDialog(hWndParent) == FALSE)			return FALSE;
#endif UDPMULTICAST

	return TRUE;
}

// This function is used for sources that have demuxes. TSReader will call this function as 
// it needs access to a PID. For devices that return the entire transport stream, this function
// can be ignored.
BOOL TSReader_PIDManagement(BOOL fAdd, int nPID, BOOL fTemporary)
{
	return TRUE;
}

// If using an interface with a demux, this function tells TSReader if the PID is currently
// active (turned on in the demux) or not.
BOOL TSReader_IsPIDActive(int nPID)
{
	return TRUE;
}

// This function returns information and cabilities about your source
BOOL TSReader_GetDescription(char * szDescription, char * szCommandLineParameters, BOOL * fCanBeStopped, int * nMaxPIDs, DWORD * dwCapabilities)
{
	if (szDescription != NULL)
	{
		lstrcpy(szDescription, TSP102_BASE_LABEL);
		lstrcat(szDescription, TSP102_TSIO_FORMAT ? TSP102_FORMAT_1 : TSP102_FORMAT_0);
	}
	if (fCanBeStopped != NULL)
		*fCanBeStopped = FALSE;
	if (nMaxPIDs != NULL)
		*nMaxPIDs = 8192;
	if (szCommandLineParameters != NULL)
		lstrcpy(szCommandLineParameters, "n/a");	
	if (dwCapabilities != NULL)
		*dwCapabilities = 0;
	return TRUE;
}

// This is where you get a chance to parse the command-line and save the parameters which
// can be used later in the TuneDialog function rather than presenting a tune dialog.
// This function is only called with the full-version of TSReader.
BOOL TSReader_ParseCommandLine(PSOURCESTRUCT ss, char * szCommandLine, BOOL fQuiet)
{
	return TRUE;
}

// This function will be used in the future for CI-CAMs
BOOL TSReader_SetChannel(int nChannel)
{
	return TRUE;
}

// This function is called at a 1Hz rate and returns a signal report string which
// TSReader then displays in it's main window
BOOL TSReader_GetSignalString(char * szString)
{
	lstrcpy(szString, szLastSignalReport);
	return TRUE;
}

// This function is called at a 1Hz rate and returns a tune report string which
// TSReader then displays in it's main window
BOOL TSReader_GetTunerString(char * szString)
{
	lstrcpy(szString, szLastTune);
	return TRUE;
}

// This function is called by TSReader when it needs to send a DiSEqC command.
// Only sources that have CAPABILITIES_DISEQC_POSITIONER will get called.
BOOL TSReader_SendDiSEqC(BYTE * bCommand, int nLength)
{
	return TRUE;
}

// Standard DLL load point. Save the hModule for later if you want to get to
// your resources
BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
    switch( ul_reason_for_call )
	{
    case DLL_PROCESS_ATTACH:
		hInstance = (HINSTANCE)hModule;
		break;
    case DLL_PROCESS_DETACH:
		break;
    }
    return TRUE;
}
